#!/bin/sh
set -e

SHORT_MIN_RUNS=20
SHORT_WARMUP=5
LONG_MIN_RUNS=5
LONG_WARMUP=1

# $1: benchmark name
# $2: command to run
# $3: warmup rounds
# $4: minimum runs
run_hyperfine() {
    NAME="$1"
    COMMAND="$2"
    WARMUP="$3"
    RUNS="$4"
    shift
    shift
    shift
    shift

    echo "Running parameter scan benchmark '$NAME'"
    hyperfine --warmup "$WARMUP" --min-runs "$RUNS" "$COMMAND" --export-csv "$NAME.csv" $@
    echo "Benchmark '$NAME' has finished, results written to '$NAME.csv'"
}

# $1: benchmark name
# $2: command to run
# $3: warmup rounds
# $4: minimum runs
# $5: parameter placeholder
# $6: parameter minimum value
# $7: parameter maximum value
run_hyperfine_scan() {
    NAME="$1"
    COMMAND="$2"
    WARMUP="$3"
    RUNS="$4"
    PARAM="$5"
    MIN="$6"
    MAX="$7"
    shift
    shift
    shift
    shift
    shift
    shift
    shift
    echo "Running parameter scan benchmark '$NAME'"
    hyperfine --warmup "$WARMUP" --min-runs "$RUNS" -P "$PARAM" "$MIN" "$MAX" "$COMMAND" --export-csv "$NAME.csv" $@
    echo "Benchmark '$NAME' has finished, results written to '$NAME.csv'"
}

run_two_counters() {
    echo "Running the Two Counters benchmarks"
    run_hyperfine_scan 'tc_tl' './tc {bits} | ./oink --no --tl' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 8 14
    run_hyperfine_scan 'tc_pmtl_opm_up' './tc {bits} | ./oink --no --pmtl -c opm,up' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 8 20
    run_hyperfine_scan 'tc_pmtl_spm_up' './tc {bits} | ./oink --no --pmtl -c spm,up' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 14 20
    run_hyperfine_scan 'tc_pmtl_opm_no_up' './tc {bits} | ./oink --no --pmtl -c opm,noup' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 14 20
    run_hyperfine_scan 'tc_pmtl_spm_no_up' './tc {bits} | ./oink --no --pmtl -c spm,noup' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 14 20
}

run_two_counters_plus() {
    echo "Running the Two Counters+ benchmarks (part 1)"
    run_hyperfine_scan 'tcp_rtl' './tc+ {bits} | ./oink --no --rtl' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 1 12
    run_hyperfine_scan 'tcp_pmtl_opm_up' './tc+ {bits} | ./oink --no --pmtl -c opm,up' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 1 7
    run_hyperfine_scan 'tcp_pmtl_spm_up' './tc+ {bits} | ./oink --no --pmtl -c spm,up' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 1 9
    run_hyperfine_scan 'tcp_pmtl_opm_no_up' './tc+ {bits} | ./oink --no --pmtl -c opm,noup' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 1 10
    run_hyperfine_scan 'tcp_pmtl_spm_no_up' './tc+ {bits} | ./oink --no --pmtl -c spm,noup' $SHORT_WARMUP $SHORT_MIN_RUNS 'bits' 1 12
    echo "Running the Two Counters+ benchmarks (part 2)"
    run_hyperfine_scan 'tcp_rtl_part_2' './tc+ {bits} | ./oink --no --rtl' $LONG_WARMUP $LONG_MIN_RUNS 'bits' 13 14
    run_hyperfine_scan 'tcp_pmtl_opm_up_part_2' './tc+ {bits} | ./oink --no --pmtl -c opm,up' $LONG_WARMUP $LONG_MIN_RUNS 'bits' 8 10
    run_hyperfine_scan 'tcp_pmtl_spm_up_part_2' './tc+ {bits} | ./oink --no --pmtl -c spm,up' $LONG_WARMUP $LONG_MIN_RUNS 'bits' 10 10
    run_hyperfine_scan 'tcp_pmtl_opm_no_up_part_2' './tc+ {bits} | ./oink --no --pmtl -c opm,noup' $LONG_WARMUP $LONG_MIN_RUNS 'bits' 11 14
    run_hyperfine_scan 'tcp_pmtl_spm_no_up_part_2' './tc+ {bits} | ./oink --no --pmtl -c spm,noup' $LONG_WARMUP $LONG_MIN_RUNS 'bits' 13 14
}

run_random() {
    if [ ! -d "rngames25" ]
    then
cat << EOM
Random games must be generated using the 'prepare_games.sh' script before this
benchmark can run. Run the script, this will create the rngames25, rngames50,
etc. folders.
EOM
        exit 1
    fi
    echo "Running the random games benchmarks"
    run_hyperfine 'rnd_pmtl_opm_up' './test_solvers --pmtl -c opm,up rngames{size}/game*.pg' $LONG_WARMUP $LONG_MIN_RUNS -L size 25,50,100,200
    run_hyperfine 'rnd_pmtl_spm_up' './test_solvers --pmtl -c spm,up rngames{size}/game*.pg' $LONG_WARMUP $LONG_MIN_RUNS -L size 25,50,100,200
    run_hyperfine 'rnd_pmtl_opm_no_up' './test_solvers --pmtl -c opm,noup rngames{size}/game*.pg' $LONG_WARMUP $LONG_MIN_RUNS -L size 25,50,100,200,2000
    run_hyperfine 'rnd_pmtl_spm_no_up' './test_solvers --pmtl -c spm,noup rngames{size}/game*.pg' $LONG_WARMUP $LONG_MIN_RUNS -L size 25,50,100,200
    run_hyperfine 'rnd_tl' './test_solvers --tl rngames{size}/game*.pg' $LONG_WARMUP $LONG_MIN_RUNS -L size 25,50,100,200,2000
    run_hyperfine 'rnd_opm' './test_solvers --gpm -c opm rngames{size}/game*.pg' $LONG_WARMUP $LONG_MIN_RUNS -L size 25
    run_hyperfine 'rnd_spm' './test_solvers --gpm -c spm rngames{size}/game*.pg' $LONG_WARMUP $LONG_MIN_RUNS -L size 25 # Not running further, size 50 takes 15 minutes
}

run_real_world() {
    if [ ! -d "real_world_games" ]
    then
cat << EOM
The real_world_games folder must be populated before this benchmark can run.
EOM
        exit 1
    fi
    echo "Running the real-world games benchmarks"
    run_hyperfine 'rw_tl' './test_solvers --tl real_world_games/*.pg' $LONG_WARMUP $LONG_MIN_RUNS
    run_hyperfine 'rw_pmtl_opm_up' './test_solvers --pmtl -c opm,up real_world_games/*.pg' $LONG_WARMUP $LONG_MIN_RUNS
    run_hyperfine 'rw_pmtl_spm_up' './test_solvers --pmtl -c spm,up real_world_games/*.pg' $LONG_WARMUP $LONG_MIN_RUNS
    run_hyperfine 'rw_pmtl_opm_no_up' './test_solvers --pmtl -c opm,noup real_world_games/*.pg' $LONG_WARMUP $LONG_MIN_RUNS
    run_hyperfine 'rw_pmtl_spm_no_up' './test_solvers --pmtl -c spm,noup real_world_games/*.pg' $LONG_WARMUP $LONG_MIN_RUNS
}

run_no_tangles() {
    echo "Running the no-tangles benchmarks (random games)"
    run_hyperfine 'rnd_pmtl_notangles_opm_no_up' './test_solvers --pmtl -c notangles,opm,noup rngames{size}/game*.pg' $LONG_WARMUP $LONG_MIN_RUNS -L size 25
}

if ! command -v hyperfine > /dev/null 2>&1
then
cat << EOM
This script uses 'hyperfine' to run its benchmarks.

Hyperfine is a command-line benchmarking tool found at
https://github.com/sharkdp/hyperfine . It may also be installed from your
system's package manager. (e.g. 'apt install hyperfine') Make sure that the
'hyperfine' executable can be found in one of the folders in your \$PATH
environment variable.
EOM
    exit 1
fi

cat << EOM
Please choose the benchmark you was to run:

1) All benchmarks
2) Two Counters
3) Two Counters+
4) Random games
5) Real-world games
6) No tangles (random games)
q) Exit
EOM

while true;
do
    printf 'Please select an option: '
    read selection
    if [ "$selection" != "${selection#[QqeEsS]}" ]; # Quit, Exit, Stop
    then
        exit 0
    fi


    if [ "$selection" = "1" ];
    then
        run_two_counters
        run_two_counters_plus
        run_random
        exit 0
    fi

    if [ "$selection" = "2" ];
    then
        run_two_counters
        exit 0
    fi

    if [ "$selection" = "3" ];
    then
        run_two_counters_plus
        exit 0
    fi

    if [ "$selection" = "4" ]; then
        run_random
        exit 0
    fi

    if [ "$selection" = "5" ]; then
        run_real_world
        exit 0
    fi

    if [ "$selection" = "6" ]; then
        run_no_tangles
        exit 0
    fi

    echo 'Unrecognized option, please try again'
done



